import configparser
import os

CONFIG_SECTION = "URLCHECKER"


def createConfiguration(
    envvar="URLCHECKER_SETTINGS", cfgfile=None, mapping=None
):
    config = configparser.ConfigParser(empty_lines_in_values=False)
    # Default values
    config[CONFIG_SECTION] = {
        "INTERVAL": -1,
        "DOMAIN": "",
        "URL": "",
        "DNS": "",
    }

    # User defined values
    if (envvar is not None) and (envvar in os.environ):
        config.read([os.environ.get(envvar)])
    if cfgfile is not None:
        config.read([cfgfile])
    if mapping is not None:
        config.read_dict({CONFIG_SECTION: mapping})

    # Required values
    for parameter in ["SQLITE"]:
        if parameter not in config[CONFIG_SECTION]:
            raise AttributeError("Config %s not defined" % parameter)

    return config[CONFIG_SECTION]


def logConfiguration(db, status_id, config):
    with db._db.atomic():
        for key, value in config.items():
            try:
                # Check previous parameter value
                previous_value = (
                    db.ConfigurationChange.select()
                    .where(db.ConfigurationChange.parameter == key)
                    .order_by(db.ConfigurationChange.status.desc())
                    .get()
                    .value
                )
            except db.ConfigurationChange.DoesNotExist:
                previous_value = None

            if previous_value != value:
                db.ConfigurationChange.create(
                    status=status_id, parameter=key, value=value
                )
