import requests
from urllib.parse import urlparse, urlunsplit
from forcediphttpsadapter.adapters import ForcedIPHTTPSAdapter


PREFERRED_TYPE = "text/html"
TIMEOUT = 2


def getUrlHostname(url):
    return urlparse(url).hostname


def getUserAgent(version):
    return "%s/%s (+%s)" % (
        "URLCHECKER",
        version,
        "https://lab.nexedi.com/romain/url-checker",
    )


def request(url, headers=None, session=requests, version=0):

    if headers is None:
        headers = {}
    if "Accept" not in headers:
        headers["Accept"] = "%s;q=0.9,*/*;q=0.8" % PREFERRED_TYPE
    if "User-Agent" not in headers:
        # XXX user agent
        headers["User-Agent"] = getUserAgent(version)

    kwargs = {}
    kwargs["stream"] = False
    kwargs["timeout"] = TIMEOUT
    kwargs["allow_redirects"] = False
    kwargs["verify"] = True
    args = ["GET", url]

    kwargs["headers"] = headers

    try:
        response = session.request(*args, **kwargs)
    except requests.exceptions.SSLError:
        # XXX Enter into unknown host
        response = requests.models.Response()
        response.status_code = 526
    except requests.exceptions.ConnectionError:
        response = requests.models.Response()
        response.status_code = 523
    except requests.exceptions.Timeout:
        response = requests.models.Response()
        response.status_code = 524
    except requests.exceptions.TooManyRedirects:
        response = requests.models.Response()
        response.status_code = 520

    return response


def logHttpStatus(db, ip, url, code, status_id):

    with db._db.atomic():
        try:
            # Check previous parameter value
            previous_entry = (
                db.HttpCodeChange.select()
                .where(
                    db.HttpCodeChange.ip == ip, db.HttpCodeChange.url == url
                )
                .order_by(db.HttpCodeChange.status.desc())
                .get()
            )
        except db.HttpCodeChange.DoesNotExist:
            previous_entry = None

        if (previous_entry is None) or (previous_entry.status_code != code):
            previous_entry = db.HttpCodeChange.create(
                status=status_id, ip=ip, url=url, status_code=code
            )
        return previous_entry.status_id


def checkHttpStatus(db, status_id, url, ip, bot_version):
    parsed_url = urlparse(url)
    hostname = parsed_url.hostname
    request_kw = {}
    # SNI Support
    if parsed_url.scheme == "https":
        # Provide SNI support
        base_url = urlunsplit(
            (parsed_url.scheme, parsed_url.netloc, "", "", "")
        )
        session = requests.Session()
        session.mount(base_url, ForcedIPHTTPSAdapter(dest_ip=ip))
        request_kw["session"] = session
        ip_url = url
    elif parsed_url.scheme == "http":
        # Force IP location
        parsed_url = parsed_url._replace(netloc=ip)
        ip_url = parsed_url.geturl()
    else:
        raise NotImplementedError("Unhandled url: %s" % url)

    response = request(
        ip_url, headers={"Host": hostname}, version=bot_version, **request_kw
    )
    logHttpStatus(db, ip, url, response.status_code, status_id)
