import socket
import errno

TIMEOUT = 2


def logNetwork(db, ip, transport, port, state, status_id):

    with db._db.atomic():
        try:
            # Check previous parameter value
            previous_entry = (
                db.NetworkChange.select()
                .where(
                    db.NetworkChange.ip == ip,
                    db.NetworkChange.transport == transport,
                    db.NetworkChange.port == port,
                )
                .order_by(db.NetworkChange.status.desc())
                .get()
            )
        except db.NetworkChange.DoesNotExist:
            previous_entry = None

        if (previous_entry is None) or (previous_entry.state != state):
            previous_entry = db.NetworkChange.create(
                status=status_id,
                ip=ip,
                transport=transport,
                port=port,
                state=state,
            )
        return previous_entry.status_id


def isTcpPortOpen(db, ip, port, status_id):
    is_open = False
    sock = socket.socket()
    sock.settimeout(TIMEOUT)
    try:
        sock.connect((ip, port))
        state = "open"
        is_open = True
    except ConnectionRefusedError:
        state = "closed"
    except socket.timeout:
        state = "filtered"
    except OSError as e:
        if e.errno == errno.EHOSTUNREACH:
            # OSError: [Errno 113] No route to host
            state = "filtered"
        elif e.errno == errno.ENETUNREACH:
            # OSError: [Errno 101] Network is unreachable
            state = "unreachable"
        else:
            sock.close()
            raise
    except:
        sock.close()
        raise

    sock.close()

    logNetwork(db, ip, "TCP", port, state, status_id)
    return is_open
